from typing import Callable, Union


def find_class_method(bases, method_name) -> Union[Callable, None]:
    for base in bases:
        if hasattr(base, method_name):
            return getattr(base, method_name)
        for _bases in base.__bases__:
            found = find_class_method(bases=_bases, method_name=method_name)
            if found is not None:
                return found
    return None



from wiederverwendbar.singleton import Singleton


class A(metaclass=Singleton):
    def __init__(self):
        print("A created")

    def __del__(self):
        print("A deleted")


class B(metaclass=Singleton):
    def __init__(self):
        print("B created")

    def __del__(self):
        print("B deleted")


class C(metaclass=Singleton):
    def __init__(self):
        print("C created")

    def __del__(self):
        print("C deleted")


def init():
    A(init=True)
    B(init=True)
    C(init=True)
    print("init")


def main():
    a = A()
    b = B()
    c = C()

    print("main")


def end():
    Singleton.delete_all()
    print("end")


if __name__ == '__main__':
    init()
    main()
    end()

    print()
